# ANALYZING THE EFFECT OF IRONY TYPE ("IMPLICATION") ON ACCEPTABILITY

# If the "ex2.critical" file is not already loaded, load it with:

ex2.critical <- read.csv("ex2.critical.csv")

# We are going to perform three similar analyses with this script.
# First, we will use the normalized Acceptability values as our dependent variable for linear regression.
# Second, we will use the original un-normalized (1-7) values and see if this makes a difference.
# Third, we will use the original values, but use ordinal logistic regression instead of linear.

# There are several different regression approaches that could work here. We will start with a mixed
# model, using a random intercept for Subject (Subjects may vary in their average Acceptability rating)
# and a random slope for the predictor of most interest, Implication (because Subjects could vary in
# their response to the four levels of that variable). Though we took care of some by-Item variation
# in the normalization process, we will still include Item as a random intercept only, since each Item
# only has one value of Implication. We will not include Group because there is no reason to think Groups
# behave as units; it would also require another layer of nesting - the model is complex enough as it is!

library(lme4)	# unless already loaded

# We would like to set the baseline level for Implication as "NonIronic", otherwise this will be set
# alphabetically. The baseline level is the one that does not appear, that is essentially zero.

ex2.critical$Implication <- relevel(ex2.critical$Implication, "NonIronic")

m1 <- lmer(Acceptability2 ~ Implication + (Implication|Subject) + (1|Item), ex2.critical)

# At least on my computer, this yields an error message saying the model did not converge in 10000
# evaluations. To increase the maximum number of evaluations, run the following:

control <- lmerControl(optCtrl = list(maxfun = 100000))

m1 <- lmer(Acceptability2 ~ Implication + (Implication|Subject) + (1|Item),
	ex2.critical, control = control)

# Fixed Effects:
#       (Intercept)  ImplicationIrony1  ImplicationIrony2  ImplicationIrony3  
#            0.1891            -0.1549            -0.2109            -0.4517  

# The fixed-effects output shows NonIronic as most acceptable, followed by Irony1, then Irony2, then
# finally Irony3, whose coefficient is -0.4517. This represents almost half the average range of an item.

# Just to check that this predictor is significant, we can run drop1().

drop1(m1, test = "Chisq")	# p = .001094

# So the key predictor, Implication (Irony Type) is significant, but not massively so, in this analysis.

# Running the raw 1-7 Likert scale Acceptability in a linear regression is not quite kosher. But let's
# try it anyway, and see what happens. Of course, we expect larger coefficients, since the scale is
# seven times larger, but the significance levels should (?) be similar.

m2 <- lmer(Acceptability ~ Implication + (Implication|Subject) + (1|Item), 
	ex2.critical, control = control)

# Fixed Effects:
#       (Intercept)  ImplicationIrony1  ImplicationIrony2  ImplicationIrony3  
#            2.1278            -0.5222            -0.8611            -0.9056 

# We do see a slight difference here. Whereas in m1, Irony1 and Irony2 were quite close togther,
# in m2, it is Irony2 and Irony3 that are very close together. Remembering that each Item represents
# 1/18 of an Irony condition, it must be that something about the normalization shifted the positioning
# -- but not the relative order -- of the four categories.

drop1(m2, test = "Chisq")	# p = 2.472e-07 = .0000002472

# Now this is a surprise. The significance level is much, much higher here. Remembering that the
# normalization worked on an item-by-item basis, and that the items are the units underlying the
# Irony type conditions, it may be that the normalization actually destroyed useful information (?).

# Finally, we will attempt a similar model using ordinal logistic regression. The basic concept of
# ordinal logistic regression is that there is no assumption of evenly-spaced steps between the values
# of the dependent variable. They might be called 1, 2, 3, 4, 5, but they might as well be called
# Tiny, Small, Medium, Big, Huge -- the point is that the ordering is known, but the spacing is not.
# The most common type of ordinal model makes an assumption called "proportional odds". This means that
# every predictor affects the response in the same way at every point along the scale. So, for example,
# if men favor women 2:1 in the "choice" between scale point 2 and scale point 1, then men will also
# favor women 2:1 in the choice between scale point 4 and scale point 4. This assumption allows the
# model to have a relatively small number of coefficients. 

library(ordinal)		# unless already loaded

# The response for ordinal logistic regression must be an "ordered factor", which I create below.
# The function for mixed-effects ordinal logistic regression is clmm(); compare clm() for ordinary models.

ex2.critical$Acceptability[ex2.critical$Acceptability == 0] <- 1
ex2.critical$Acceptability3 <- factor(ex2.critical$Acceptability, ordered = T, levels = 1:7)

m3 <- clmm(Acceptability3 ~ Implication + (Implication|Subject) + (1|Item), ex2.critical)

# This model takes quite a long time to run! It could be a problem with larger data sets. The output
# is somewhat different and contains additional intercepts, but we focus again on the fixed effects.

# Coefficients:
# ImplicationIrony1 ImplicationIrony2 ImplicationIrony3 
#            -1.252            -3.067            -3.464 

# The order is again the same, the spacing appears to be intermediate between the previous models.
# drop1() appear to give an error with this model. We can instead fit a null model:

m3.null <- clmm(Acceptability3 ~ 1 + (Implication|Subject) + (1|Item), ex2.critical)

anova(m3.null, m3)		# p = 3.514e-10 = .0000000003514

# Interestingly, the ordinal logistic model, which is thought of as less powerful because it ignores
# all information about the spacing of levels, has returned by far the most significant p-value. Not
# coincidentally, the magnitude of the coefficients is also much larger.

# Some aspect of these differences may relate to the skewedness of the data we have been working with.
# Fully 87.5% of the "critical" acceptability ratings are either 1 or 2, with only 12.5% rqnging from
# 3 to 7. It may be that the ordinal method makes better use of the scarce higher values.

# Finally, whenever we have a key predictor with 3 or more categories, we often want to know about
# the significance of differences between just two of the levels. For example, we might want to know
# if there is a significant differnece between Irony2 (-3.067) and Irony3 (-3.464) here.

# There are several ways to accomplish this statistically, including various 'post hoc' tests. One of
# the simplest ways to accomplish it is to fit a new model where the two levels in question are combined,
# and compare this to a model where the two levels are kept separate. However, when the predictor occurs
# as a random slope as well as a fixed effect, the proper way to do this becomes unclear...

ex2.critical$Impliczation2 <- ex2.critical$Implication
levels(ex2.critical$Implication2)
# "NonIronic" "Irony1"    "Irony2"    "Irony3"
levels(ex2.critical$Implication2) <- c("NonIronic", "Irony1", "Irony23", "Irony23")

m3 <- clmm(Acceptability3 ~ Implication + (Implication|Subject) + (1|Item), ex2.critical)
m4 <- clmm(Acceptability3 ~ Implication2 + (Implication|Subject) + (1|Item), ex2.critical)
anova(m3, m4) # p = .5374

m3 <- clmm(Acceptability3 ~ Implication + (Implication|Subject) + (1|Item), ex2.critical)
m5 <- clmm(Acceptability3 ~ Implication2 + (Implication2|Subject) + (1|Item), ex2.critical)
anova(m3, m5) # p = .9267

# Either way, we cannot conclude that there is a significant difference between Irony2 and Irony3.
# It is probably more likely that a significant difference would emerge between Irony1 and Irony2,
# or between NonIronic and Irony1. The demonstration of this is left as an exercise!

# THE END

