# Script note: Any line beginning with # is a comment. You can run these lines and nothing will happen.
# Lines not beginning with # can be "run" in the console. If you are using a Mac, simply press Command-
# Enter with the cursor on such a line, and it will be executed. If you are on a Windows computer, use
# Control-R instead, to run any line of code in this script. You can also cut and paste multiple lines.

# ANALYSIS OF LYONNAIS ORIGINAL [kl, gl] SEQUENCES WITH ORDINARY (FIXED-EFFECTS) REGRESSION MODELS

# The data is in the file "lyon_l.csv". I have placed this file online, so you can open it as follows:

data <- read.csv("http://www.danielezrajohnson.com/lyon_l.csv")

# It's also part of the .zip file, so once extracting that you can open it like this, navigating to
# the directory which you've saved it in:

data <- read.csv(file.choose())

# Using the nrow() command, we can see how many tokens there are in this file.

nrow(data)	# 157

# So this is a small data set, which is a subset of larger data collected by Jonathan Kasstan.
# The dependent variable, called "L", is either a "alveolar.lateral" [l] or a "palatal.glide" [j].
# We choose [j] as the "application value" meaning that positive effects favor e.g. [kjar] over [klar].

# If we use the names() command, we will see all the column names for this data file.

names(data)

#  [1] "FP.item"               "Arp.item"              "French.item"          
#  [4] "POS"                   "L"                     "Stress"               
#  [7] "Environment"           "Preceding.seg"         "Preceding.seg.feature"
# [10] "Following.seg"         "Following.seg.feature" "Speaker"              
# [13] "Sex"                   "Age"                   "Speaker.type"         
# [16] "Born"                  "Residence"             "Style"                
# [19] "Interview.type"        "Sample.site" 

# However, for the present exercise, we are only going to look at four external (social) predictors:
# Sex, Age, Born, Style

# And we are going to look at four internal predictors:
# POS, Stress, Preceding.seg (i.e. [k] or [g]), Following.seg (i.e. which vowel follows)

# We will add predictors by fitting two "nested" models, and establish the significance of the added
# predictor using the anova() function. In this case, "nested" models are models that are identical
# except one of them has one more predictor than the other.

# We will start with a basic "null model" that simply predicts the average rate of [j] overall.

m0 <- glm(L == "palatal.glide" ~ 1, data, family = binomial)

# Now we will fit models that include the four external predictors.

m1 <- glm(L == "palatal.glide" ~ Sex, data, family = binomial)

# If we simply enter

m1

# we will see that the coefficient "Sex_male" is -0.185. This is a small Sex effect size.

# To test the statistical significance of the Sex effect, we use the anova() command.

anova(m0, m1, test = "Chisq")

# The p-value of 0.7219 shows that the Sex effect is clearly not statistically significant.

# We will repeat the same procedure for the other external predictors, Age, Born, and Style.

m2 <- glm(L == "palatal.glide" ~ Age, data, family = binomial)
anova(m0, m2, test = "Chisq")		# p = 0.6069, not statistically significant

m3 <- glm(L == "palatal.glide" ~ Born, data, family = binomial)
anova(m0, m3, test = "Chisq")		# p = 1.069e-13 = 0.0000000000001069, very statistically significant!

m4 <- glm(L == "palatal.glide" ~ Style, data, family = binomial)
anova(m0, m4, test = "Chisq")		# p = 2.045e-08 = 0.00000002045, very statistically significant

# So by testing each external predictor in turn, we find a big difference between Sex and Age,
# which do NOT have a significant effect on the realization of the variable, and Born and Style,
# which have a very clearly significant effect on it.

# Although we concluded that Age was not significant, it is worth looking at the output of model m2,
# in order to see how a continuous (numeric) predictor works in the model.

m2

# Coefficients:
# (Intercept)          Age  
#     1.32114     -0.01509  

# This model means that for each year of increasing age (the range is from 63 to 84), the log-odds
# of the palatal [j] variant decreases by 0.01509 log-odds. Of course, we don't want to make anything
# of this (is the trend even in the right direction?) because interpreting such a non-significant 
# predictor is pointless, since we've already concluded the result could be due to mere chance.

# If we examine the model m3, we see a different kind of output that needs some explanation:

m3

# Coefficients:
#   (Intercept)   BornRontalon  BornSt.Martin     BornYzeron  
#        -2.398          4.883          2.671          4.963  

# What we have to remember here is that there is a missing level, a default level that is set to zero.
# This is the "BornCoise" level which is first alphabetically. "Coise" is set to zero, and compared to
# Coise, the other three locations are much more favorable to a palatal glide [j] in the historical
# cluster. Adding the intercept, we see that the log-odds of [j] in Coise is -2.398 (strongly
# disfavoring [j]), while the log-odds of [j] in the other three locales are roughly between 0 and 2.5.
# This may make sense geographically since Coise is located further west from the other three places.

# If we examine the model m4, we can see the output for the Style predictor.

m4

# Coefficients:
#   (Intercept)   Stylereading  Stylewordlist  
#        -2.197         17.763          2.795  

# Again, remembering that the "conversation" style, being first alphabetically, is set to zero, we see
# that conversation style disfavors the [j] variant, while adding the "wordlist" coefficient brings us
# to a much more favorable 0.598 log-odds (-2.197 + 2.795). The very high coefficient for "reading" is
# suspicious. Such large numbers usually meant that there is no variation in that cell. Let's check.

xtabs(~L + Style, data)

#                   Style
# L                  conversation reading wordlist
#   alveolar.lateral           27       0       44
#   palatal.glide               3       3       80

# The breakdown in wordlist style is 80 vs. 44, which in log-odds is:

log(80/44)		# 0.598

# So we can see how the regression function accurately estimates the proportions here!
# However, the reading data is only based on three tokens. Certainly, we would not want to estimate
# a "range" for the predictor Style based on this data. Reading seems to favor [j], but we don't know.

# In my opinion (and this is disputed), the p-value is the most reliable way to assess the relative
# importance of any two predictors, not the range of effect coefficients.

# A related point (not illustrated here) is that it is almost impossible to compare models that have
# been fit to different data sets, especially of different sizes. However, this is frequently done!

# It now makes sense to build a model with both significant external predictors, Born and Style.
# We can then use the drop1() function to check that both are significant, while taking
# account of the other.

m5 <- glm(L == "palatal.glide" ~ Born + Style, data, family = binomial)
drop1(m5, test = "Chisq")

# The p-values here confirm that both "Born" and "Style" are highly significant predictors, even
# when both are taken into account at the same time.

# Now we will repeat the analogous process for the four internal (linguistic) predictors, which are
# POS (part of speech), Stress, Preceding.seg, and Following.seg.

m6 <- glm(L == "palatal.glide" ~ POS, data, family = binomial)
anova(m0, m6, test = "Chisq")		# p = 5.146e-06 = 0.000005146 = very statistically significant

m7 <- glm(L == "palatal.glide" ~ Stress, data, family = binomial)
anova(m0, m7, test = "Chisq")		# p = 0.6809 = NOT statistically significant

m8 <- glm(L == "palatal.glide" ~ Preceding.seg, data, family = binomial)
anova(m0, m8, test = "Chisq")		# p = 0.7081 = NOT statistically significant

m9 <- glm(L == "palatal.glide" ~ Following.seg, data, family = binomial)
anova(m0, m9, test = "Chisq")		# p = 0.00121 = marginally statistically significant

# These univariate analyses have returned the conclusion that among the internal predictors,
# part of speech and following segment have a significant effect on the response variable. However,
# stress and the identity of the preceding segment (always [k] or [g]) does not make a difference.

# If we want to know what the effect of e.g. part of speech or following segment is, we would look at
# the corresponding models, either m6 or m9, or in fact m10 below.

# Setting up a multiple regression with the two internal predictors and the two external ones,
# we can test if their significance remains when the other predictors are in the model.

m10 <- glm(L == "palatal.glide" ~ Born + Style + POS + Following.seg, data, family = binomial)

drop1(m10, test = "Chisq")

# The drop1() output shows p-values that are higher than the univariate analyses above. This is normal.
# When several predictors are included, they tend to overlap to some extent and reduce the significance
# of any given predictor. Note that all the predictors still meet the usual threshold of statistical
# significance, except following segment (p = 0.139). We will remove Following.seg from our final model.

m.final <- glm(L == "palatal.glide" ~ Born + Style + POS, data, family = binomial)
 
# The output of this model is a little crazy, with extreme coefficients related to invariant cells.
# However, we can tentatively conclude from this model and those above that the glide [j] is favored by:

# being born in Rontalon and Yzeron and to a lesser degree St. Martin (and very much disfavored in Coise)
# wordlist style > conversational style, reading style unsure
# Part of Speech: adjective > common noun > proper noun > verb (less sure about the latter two)

# In the next exercise, we will take into account something we have ignored so far, the possible effects
# of individual Speaker and Word, using a mixed-effects model. This will change our results.

# Always remember: when a predictor comes out as non-significant this DOES NOT MEAN that it really does
# not influence the response variable. It simply means that an effect as large as the one observed
# could have occurred simply by random chance. In particular, with small data sets, like the one here,
# effects have to be large to be statistically significant. But that doesn't mean they might not be real.