# Script note: Any line beginning with # is a comment. You can run these lines and nothing will happen.
# Lines not beginning with # can be "run" in the console. If you are using a Mac, simply press Command-
# Enter with the cursor on such a line, and it will be executed. If you are on a Windows computer, use
# Control-R instead, to run any line of code in this script. You can also cut and paste multiple lines.

# ANALYSIS OF LYONNAIS ORIGINAL [kl, gl] SEQUENCES WITH MIXED-EFFECTS REGRESSION MODELS

# You won't have to re-load the data if you haven't shut down R, but if you have closed R, you'll need to reload the data file. Also, any packages will need to be reloaded, though not re-installed.

# The data is in the file "lyon_l.csv". I have placed this file online, so you can open it as follows.

data <- read.csv("http://www.danielezrajohnson.com/lyon_l.csv")

# It's also part of the .zip file, so once you've extracted that you can locate and open it like this:

data <- read.csv(file.choose())

# If we use the names() command, we will remember all the column names for this data file.

names(data)

#  [1] "FP.item"               "Arp.item"              "French.item"          
#  [4] "POS"                   "L"                     "Stress"               
#  [7] "Environment"           "Preceding.seg"         "Preceding.seg.feature"
# [10] "Following.seg"         "Following.seg.feature" "Speaker"              
# [13] "Sex"                   "Age"                   "Speaker.type"         
# [16] "Born"                  "Residence"             "Style"                
# [19] "Interview.type"        "Sample.site" 

# In the previous script, we determined that the three predictors of "Born" (one of four villages where
# the informant was born), "Style" (conversation, reading, and wordlist), and "POS" (part of speech:
# adjective, common noun, proper noun, verb) all had a significant effect on the dependent variable.

m.final <- glm(L == "palatal.glide" ~ Born + Style + POS, data, family = binomial)

# A tentative conclusion from this multiple regression was that the following things favored [j],
# for example [kjar] rather than [klar].

# being born in Rontalon and Yzeron and to a lesser degree St. Martin (and very much disfavored in Coise)
# wordlist style > conversational style, reading style unsure
# Part of Speech: adjective > common noun > proper noun > verb (less sure about the latter two)

# The following command excerpt shows the estimated significance levels of each predictor.
# The expression "Pr(>Chi)" is the p-value. It is the chance that an effect as large as that observed
# (or larger) for Born (etc.) would occur if there was really no true Born effect at all. That is the
# classic frequentist meaning of a p-value, and it should be born in mind. It is often misunderstood.

drop1(m.final, test = "Chisq")

# L == "palatal.glide" ~ Born + Style + POS
#        Df Deviance    AIC    LRT  Pr(>Chi)    
# <none>      110.61 128.61                     
# Born    3   161.09 173.09 50.485 6.299e-11 ***
# Style   2   128.97 142.97 18.363 0.0001029 ***
# POS     3   131.74 143.74 21.134 9.876e-05 ***

# Now we will take into account possible effects of Speaker and Word. As a general rule, we expect the
# use of a Speaker intercept to decrease the significance of between-speaker predictors, meaning any
# predictors that are constant within a speaker's data and vary between speakers. Here, this is Born.

# Similarly, we expect the use of a Word random intercept to decrease the significance of predictors that
# are constant for each Word and vary only between words. POS should be like this.

# The predictor Style is a bit different. It varies within, not between, speakers (each speaker was
# asked to produce all three styles). With respect to word, although each word COULD have occurred
# in all three styles -- in some ways, this would have been ideal -- in fact almost all the words
# actually appear in one style only. This makes Style much more like POS, and we expect a Word intercept
# to affect the significance of Style more than the Speaker intercept does.

# To run mixed-effects models (mixed models) it is necessary to do a one-time installation of the lme4
# package. Packages are extensions to R that have been designed to carry out particular functions.
# There may be a problem if you do not have permission to install software on the computer you're using!

install.packages("lme4", dependencies = T)

# This command may ask you to select a download server from among many countries. It doesn't matter.
# If the install.packages() command works, you shouldn't have to do it again. If it doesn't work,
# you may have to quite R, right-click and run R as an administrator, and try again. However, you may
# also need to know an administrator password to make this work. Hopefully we have installed it already.

# In order to load the lme4 package and make its functions lmer() and glmer() usable, run the following:

library(lme4)

# Now we are ready to use the glmer() function, which is parallel to the glm() function. We'll fit the
# same model indicating "mm" for mixed model instead of "m", as before. First, we'll just add a random
# intercept for speaker -- indicated by (1|Speaker) -- and see what happens.

mm.speaker <- glmer(L == "palatal.glide" ~ Born + Style + POS + (1|Speaker), data, family = binomial)

# This model converges quickly -- not always the case for mixed models -- and we can use drop1()
# to test the significance of the three fixed-effect terms, in order to compare to what we saw above.

drop1(mm.speaker, test = "Chisq")

# Now let's compare the significance level for each predictor, compared to the previous model.

# Predictor		Fixed-Effects p-value		Mixed-Effects p-value (with Speaker random intercept)
# Born			.000000000063				.09
# Style			.0001						.00000073
# POS			.000099						.0003

# Now clearly all three have changed, with Style in fact becoming more significant than it was before
# (this can happen, in fact, almost everything can happen in regression analysis!)
# But it is the "Born" predictor, as expected, that has lost the most significance.
# In fact, the p-value has become over nine orders of magnitude higher, going from a conventionally
# significant (indeed, extremely significant in those statistical traditions that refer to such things)
# to a non-significant result (p = .09). What can account for this drastic shift? And how can we be
# sure which output is more correct? You can decide for yourself by comparing these two commands:

xtabs(~L + Born, data)				# this is the fixed-effect model's "point of view"

xtabs(~L + Speaker + Born, data)	# this is the mixed model's "point of view"

# You can see the details there, but the basic point is that from each village ("Born"), a small number
# of speakers were interviewed. Or at least, a small number of speakers produced any tokens of the
# variable we are concerned with. In Coise and Rontalon, it was 3 speakers each. In St. Martin, it was
# 8 speakers. And in Yzeron, it was only one speaker who produced such a high fraction of [j].

# The second point is that while we are investigating differences BETWEEN villages, there is clear
# evidence that speakers from the same village differ from one another. For example, in St. Martin,
# several speakers were roughly evenly split between [l] and [j], but one speaker produced 10 tokens
# of [j] and none of [l].

# In any statistical comparison, the more within-group variation there is, the harder it is to prove
# that there is between-group variation. In general, if we had something like 10-15 speakers from each
# of the four villages, and each person produced a considerable amount of data, then we could draw firm
# conclusions. In the present analysis, we must take the high scores of Rontalon and Yzeron and the low
# score of Coise with a major grain of salt, because of the small number of speakers underlying them.

# The Speaker random intercept allows the model to account for speakers who differ in their overall use
# of the dependent variable. For example, in St. Martin, the speaker who used 100% [j] diverged from
# the norm of the group he was part of.

# By adding the Word random intercept, we are allowing the same thing for words. We are allowing for the
# possibility that individual words might behave differently, not just as a result of the larger groups
# (e.g. Part of Speech) that they are part of.

mm.speaker.word <- glmer(L == "palatal.glide" ~ Born + Style + POS + (1|Speaker) + (1|French.item), 
	data, family = binomial)
	
# If you examine the coefficients from these mixed models, you will see that they don't change very much.
# The significance of predictors does change. In other words, we are observing large differences between
# groups, but considering by-speaker and/or by-word variation, we are coming to doubt these differences.

drop1(mm.speaker.word, test = "Chisq")

# Predictor		Fixed p-value				Mixed p-value (Speaker)			Mixed p-value (Speaker, Word)
# Born			.000000000063				.09								.19
# Style			.0001						.00000073						.00067
# POS			.000099						.0003							.21

# As anticipated, the previously-significant POS predictor has not survived the introduction of a
# random intercept for word. Because most of the POS categories only have a few words in them, any 
# within-group (individual-word) variation signals that the sample is too small for comparison.

# At first, it might be disappointing that the mixed model "makes predictors not significant". However, 
# it is better to think of the mixed model as removing predictors that should never have been considered
# significant in the first place. At least, we recognize that we don't have enough data to say they are.

# Always remember: when a predictor comes out as non-significant this DOES NOT MEAN that it really does
# not influence the response variable. It simply means that an effect as large as the one observed
# could have occurred simply by random chance. In particular, with small data sets, like the one here,
# effects have to be large to be statistically significant. But that doesn't mean they might not be real.
